import os
from http.client import responses
from urllib.parse import unquote

import requests
import json

from tqdm import tqdm


def fetch_ofl_list_json(api_url="https://api.github.com/repos/google/fonts/contents/ofl"):
    """
    :param api_url:
    :return:
    {
        "name": "abeezee",
        "path": "ofl/abeezee",
        "sha": "83498679c7af31b3e2f28d05e812f6f81c9fe4f6",
        "size": 0,
        "url": "https://api.github.com/repos/google/fonts/contents/ofl/abeezee?ref=main", # WE NEED THIS!
        "html_url": "https://github.com/google/fonts/tree/main/ofl/abeezee",
        "git_url": "https://api.github.com/repos/google/fonts/git/trees/83498679c7af31b3e2f28d05e812f6f81c9fe4f6",
        "download_url": null,
        "type": "dir",
        ...
    }
    """
    headers = {"Authorization": f"token {os.getenv("ACCESS_TOKEN")}"}
    response = requests.get(api_url, headers=headers)
    if response.status_code == 200:
        json_data = json.loads(response.text)
        return json_data
    else:
        print(f"Failed to fetch github with token: {os.getenv('ACCESS_TOKEN')}")
        print(f"Please set github token with: `googlefonts config --token your_token`")
        print(f"Or you can temporary use github token with: `googlefonts list --token your_token`")
        exit(1)


def get_ttf_download_url_list_json(url_content):
    """

    :param url_content:
    [
        {
            "path": "ABeeZee-Italic.ttf",
            "mode": "100644",
            "type": "blob",
            "sha": "da4bf4754d583f833e0dd276094697275c0a53d2",
            "size": 47012,
            "url": "https://api.github.com/repos/google/fonts/git/blobs/da4bf4754d583f833e0dd276094697275c0a53d2"
        },
        {
            "path": "ABeeZee-Regular.ttf",
            "mode": "100644",
            "type": "blob",
            "sha": "2ecf01bdec7b6a8568825da840897f9b0aee8a86",
            "size": 46016,
            "url": "https://api.github.com/repos/google/fonts/git/blobs/2ecf01bdec7b6a8568825da840897f9b0aee8a86"
        }
        ...
    ]

    :return:
    """
    download_url_list = []
    if url_content:
        for item in url_content:
            if ".ttf" in str(item["path"]):
                download_url_list.append(item["download_url"])
        return download_url_list
    else:
        print("\033[31mFailed to get download urls. Please retry!\033[0m")
        exit(-1)


def get_font_names(ofl_font_json_list):
    font_names = []
    for font_json in ofl_font_json_list:
        font_names.append(font_json["name"])
    return font_names


def fetch_ttf_url_download_list_by_name(ofl_list, font_name, force=False):
    url = None
    if force:
        url = f"https://api.github.com/repos/google/fonts/contents/ofl/{font_name}?ref=main"
    else:
        for ofl in ofl_list:
            if ofl["name"] == font_name:
                print(f"font_name: {font_name}")
                print(f"name: {ofl['name']}")
                url = ofl["url"]
                print(f"url: {url}")
    if url is None:
        raise Exception(f"Can not find url for font name: {font_name}")

    tqdm.write(f"Fetching fonts {font_name} from formulas")
    headers = {"Authorization": f"token {os.getenv("ACCESS_TOKEN")}"}
    resp = requests.get(url, headers=headers)
    if resp.status_code == 404:
        print(f"\033[31mFailed to get download urls for {font_name}. Please retry!\033[0m")
        exit(-1)
    url_content = json.loads(resp.text)
    tqdm.write(f"Successfully fetched fonts {font_name} from formulas")
    download_list = []
    for download in get_ttf_download_url_list_json(url_content):
        name = unquote(download.split("/")[-1])
        download_url = download
        download_list.append({
            "name": name,
            "download_url": download_url,
        })

    if download_list:
        return download_list
    else:
        exit(-1)


if __name__ == '__main__':
    ttf_download_list = fetch_ttf_url_download_list_by_name("abeezee", True)
    print(ttf_download_list)



import os
import platform
from typing import List

import typer
from dotenv import load_dotenv, set_key
from rich.console import Console
from google_fonts.font.list import list_fonts
from .font.install import install_fonts, install_all_fonts

app = typer.Typer()
console = Console()
env_file_path = os.path.expanduser("~/.config/google-fonts/.env")
load_dotenv(env_file_path)


@app.command(name="install", help="Install specified fonts.")
def install_fonts_cli(names: List[str] = typer.Argument(None, help="Names of the fonts to process."),
                      install_all: bool = typer.Option(False, "--all", help="Install all available fonts."),
                      force: bool = typer.Option(False, "--force/--no-force",
                                                 help="Force installation of fonts. Using name in https://github.com/google/fonts/tree/main/ofl/"),
                      github_token: str = typer.Option(None, "--token", help="GitHub Access Token for authentication."),
                      ):
    if github_token is not None:
        os.environ["ACCESS_TOKEN"] = github_token
    if install_all:
        print("Installing all available fonts...")
        install_all_fonts()
        return
    if force:
        print("Forcing installation of fonts...")
        install_fonts(names, force=force)
        return

    install_fonts(names)
    if platform.system() == "Linux":
        os.system("fc-cache -fv")


@app.command(name="list",
             help="List all available fonts. See https://github.com/google/fonts/tree/main/ofl and https://fonts.google.com/")
def list_fonts_cli(
        name: str = typer.Argument(None, help="Name of the font to list."),
        github_token: str = typer.Option(None, "--token", help="GitHub Access Token for authentication.")
):
    if github_token is not None:
        os.environ["ACCESS_TOKEN"] = github_token
    list_fonts(search_content=name)


@app.command(name="config", help="Configure GitHub Access Token.")
def config(
        github_token: str = typer.Option("", "--token", help="GitHub Access Token for authentication.")
):
    # 确保目录存在
    os.makedirs(os.path.expanduser("~/.config/google-fonts"), exist_ok=True)

    if github_token:
        # 保存 token 到 .env 文件
        set_key(os.path.expanduser("~/.config/google-fonts/.env"), "ACCESS_TOKEN", github_token)
        console.print(f"GitHub Access Token saved to .config/google-fonts/.env", style="green")
    else:
        # 如果没有提供 Token，读取并显示现有的 Token
        token = os.getenv("ACCESS_TOKEN", "")
        if token:
            console.print(f"Current GitHub Access Token: {token}", style="yellow")
        else:
            console.print("No GitHub Access Token found.", style="red")


@app.command(name="author", help="This package is written by desonglll. See my github on github.com/desonglll.")
def author():
    print("This package is written by desonglll.")
    print("See my github on github.com/desonglll.")
    pass


if __name__ == "__main__":
    app()



from google_fonts.utils.font_data import get_font_names, fetch_ofl_list_json


def list_fonts(search_content: str = None):
    # 获取字体列表
    ofl_list = fetch_ofl_list_json()
    all_font_names = get_font_names(ofl_list)

    # 模糊匹配搜索
    if search_content is not None:
        all_font_names = [name for name in all_font_names if search_content.lower() in name.lower()]

    # 如果没有匹配结果
    if not all_font_names:
        print(f"No fonts found matching '{search_content}'")
        return

    # 设置每列的固定宽度
    column_width = max(len(name) for name in all_font_names) + 2  # 动态计算宽度，+2 用于额外间距

    # 按行打印，每行 5 个字体
    for i in range(0, len(all_font_names), 5):
        # 使用字符串的 ljust() 方法按固定宽度对齐
        print("\t".join(name.ljust(column_width) for name in all_font_names[i:i + 5]))


if __name__ == '__main__':
    list_fonts()



import os
import platform
import time
from pathlib import Path

import requests
from shutil import copyfileobj
from tqdm import tqdm
from google_fonts.utils.font_data import fetch_ttf_url_download_list_by_name, fetch_ofl_list_json, get_font_names


def download_and_install_font(font_name, font_url, install_dir, max_retries=3, retry_delay=2):
    """
    Download and install a font from the given URL with a progress bar and automatic retries.
    """
    retry_count = 0

    while retry_count < max_retries:
        try:
            tqdm.write(f"Downloading {font_name} (Attempt {retry_count + 1}/{max_retries})...")
            response = requests.get(font_url, stream=True, timeout=10)
            response.raise_for_status()

            # Get total file size from headers
            total_size = int(response.headers.get('content-length', 0))

            # Save the font file locally
            font_path = os.path.join(install_dir, f"{font_name}")  # Use .otf if necessary
            with open(font_path, "wb") as f, tqdm(
                    desc=f"Installing {font_name}",
                    total=total_size,
                    unit='B',
                    unit_scale=True,
                    unit_divisor=1024,
            ) as bar:
                for chunk in response.iter_content(chunk_size=1024):
                    f.write(chunk)
                    bar.update(len(chunk))

            tqdm.write(f"{font_name} installed successfully at {install_dir}!")
            return  # Exit function on success
        except requests.exceptions.RequestException as e:
            retry_count += 1
            tqdm.write(f"Error downloading {font_name}: {e}")
            if retry_count < max_retries:
                tqdm.write(f"Retrying in {retry_delay} seconds...")
                time.sleep(retry_delay)
            else:
                tqdm.write(f"Failed to install {font_name} after {max_retries} attempts.")
                break


system = platform.system()


def get_install_dir():
    if system == "Linux":
        install_dir = Path.home() / ".fonts"
    elif system == "Darwin":  # macOS
        install_dir = Path.home() / "Library/Fonts"
    elif system == "Windows":
        install_dir = Path(os.environ.get('LOCALAPPDATA', '')) / 'Microsoft' / 'Windows' / 'Fonts'
    else:
        raise OSError(f"Unsupported operating system: {system}")
    return install_dir


API_URL = "https://api.github.com/repos/google/fonts/contents/ofl"


def install_fonts(names: list[str], force=False):
    """
    Install predefined fonts.
    """
    # 根据操作系统确定字体安装目录
    install_dir = get_install_dir()
    all_list_to_download = []
    # 创建字体目录（如果不存在的话）
    os.makedirs(install_dir, exist_ok=True)

    tqdm.write("Fetching fonts formulas from https://api.github.com/repos/google/fonts/contents/ofl")
    ofl_list = fetch_ofl_list_json(API_URL)
    tqdm.write("Successfully fetched fonts formulas")

    all_font_names = get_font_names(ofl_list)

    for name in names:
        if name not in all_font_names and not force:
            print(f"\033[31m{name}\033[0m not found in ofl_list")
            print("\033[33mUsing following command to see font list: `google-fonts list`\033[0m")
            return exit(-1)
        for item in fetch_ttf_url_download_list_by_name(ofl_list, name, force=force):
            all_list_to_download.append(item)
    for item in all_list_to_download:
        download_and_install_font(item["name"], item["download_url"], install_dir)


def install_all_fonts():
    ofl_list = fetch_ofl_list_json()
    all_fonts = get_font_names(ofl_list)
    install_fonts(all_fonts)
    if system == "Linux":
        # 刷新字体缓存
        os.system("fc-cache -f -v")
